package com.sysdevmobile.zebrahudinterface;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.InsetDrawable;
import android.media.ExifInterface;
import android.view.Gravity;
import android.view.View;

import com.symbol.zebrahud.ZebraHud;

import com.sysdevsolutions.external.kclientv50.KExternalEventsHandler;
import com.sysdevsolutions.external.kclientv50.KExternalEventsInterface;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class HudInterface {
    static private ZebraHud mHud=null;
    static private byte[] cachedImage = null;
    static private String mImageFile="";
    static private boolean mGetNewImage = false;
    static private View mPreviewView = null;
    static private KExternalEventsInterface mKEvents = null;
    static private Activity mActivity = null;
    static private KExternalEventsHandler mKEventsListener = null;
    static private Object mCameraSyncObject = null;
    static private AtomicBoolean mCameraSyncResult = new AtomicBoolean();
    static private Socket mCastSocket = null;
    static private String mCastServer = "";
    static private int mCastPort = 0;
    static private ReentrantLock mCastLock = new ReentrantLock();


    static public void Initialize(Activity act, Context ctx, KExternalEventsInterface kEvents) throws Exception
    {
        mActivity = act;
        mKEvents=kEvents;
        mHud = new ZebraHud();

        mKEventsListener = new KExternalEventsHandler() {
            @Override
            public void onAppPaused(Context ctx) {
                if(mActivity!=null && mHud!=null)
                    mHud.onPause(mActivity);
            }

            @Override
            public void onAppResumed(Context ctx) {
                if(mActivity!=null && mHud!=null)
                    mHud.onResume(mActivity, new ZebraHud.EventListener() {
                        @Override
                        public void onConnected(Boolean connected) {
                            if (connected) mHud.showCachedImage(cachedImage); // handle hud disconnect/reconnect
                        }

                        @Override
                        public void onImageUpdated(byte[] image) {
                            final byte[] imageF = image;
                            cachedImage = imageF; // handle hud disconnect/reconnect
                            if(mCastSocket != null)
                            {
                                Thread thread = new Thread() {
                                    @Override
                                    public void run() {
                                        CastImage(imageF);
                                    }
                                };

                                thread.start();
                            }
                        }

                        @Override
                        public void onCameraImage(Bitmap bitmap) {
                            onCameraImageHandler(bitmap);
                        }
                    });
            }
        };
        kEvents.AddOnAppEventListener(mKEventsListener);

        mHud.onStart(act);
        mHud.onResume(act, new ZebraHud.EventListener() {
            @Override
            public void onConnected(Boolean connected) {
                if (connected) mHud.showCachedImage(cachedImage); // handle hud disconnect/reconnect
            }

            @Override
            public void onImageUpdated(byte[] image) {
                final byte[] imageF = image;
                cachedImage = imageF; // handle hud disconnect/reconnect
                if(mCastSocket != null)
                {
                    Thread thread = new Thread() {
                        @Override
                        public void run() {
                            CastImage(imageF);
                        }
                    };

                    thread.start();
                }
            }

            @Override
            public void onCameraImage(Bitmap bitmap) {
                onCameraImageHandler(bitmap);
            }
        });
    }

    static public void DeInitialize() throws Exception
    {
        if(mKEvents != null && mKEventsListener != null)
            mKEvents.RemoveOnAppEventListener(mKEventsListener);
        mKEventsListener=null;

        if(mHud!=null) {
            mHud.clearDisplay();
            if (mActivity != null) {
                mHud.onPause(mActivity);
                mHud.onStop(mActivity, false);
            }
        }
        mHud = null;
        mPreviewView=null;
        mKEvents=null;
        mActivity=null;
    }

    static private void onCameraImageHandler(Bitmap bitmap) {
        if(mPreviewView != null)
        {
            double vW = mPreviewView.getLayoutParams().width;
            double vH = mPreviewView.getLayoutParams().height;
            double xRatio = vW / ((double)bitmap.getWidth());
            double yRatio = vH / ((double)bitmap.getHeight());

            double ratio = Math.min(xRatio, yRatio);

            double bW=((double)bitmap.getWidth())*ratio;
            double bH=((double)bitmap.getHeight())*ratio;

            int wDifference = (int)(vW - bW) / 2;
            int hDifference = (int)(vH - bH) / 2;

            BitmapDrawable bd = new BitmapDrawable(bitmap);
            bd.setGravity(Gravity.FILL);
            InsetDrawable id = new InsetDrawable(bd, wDifference, hDifference, wDifference, hDifference);

            mPreviewView.setBackground(id);
        }
        if(mGetNewImage && mImageFile.length()>0)
        {
            try {
                mGetNewImage = false;

                File file = new File(mImageFile);
                file.createNewFile();
                FileOutputStream fos = new FileOutputStream(file);
                bitmap.compress(Bitmap.CompressFormat.JPEG, 100, fos);
                fos.flush();
                fos.close();
                mGetNewImage = false;
                if(mCameraSyncObject!=null)
                {
                    synchronized (mCameraSyncObject)
                    {
                        mCameraSyncResult.set(true);
                        mCameraSyncObject.notify();
                    }
                }
                //mKEvents.NotifyTopForm(new String[]{"NewHUDImage"});

            } catch (Exception e) {

            }

        }
    }

    static public void SetDisplayOn(int setOn) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.setDisplayOn(setOn != 0);
    }

    static public void SetMicrophoneOn(int setOn) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.setMicrophoneEnabled(setOn != 0);
    }

    static public void SetBrightness(int brightness) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.setBrightness(brightness);
    }

    static public void ClearDisplay() throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.clearDisplay();
    }

    static public int IsConnected() throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        return mHud.isConnected() ? 1 : 0;
    }

    static public void StartCamera() throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mPreviewView = null;
        mHud.setCameraEnabled(true);
        mHud.startCameraCapture();
    }

    static public void StartCameraWithPreview(View v) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mPreviewView = v;
        mHud.setCameraEnabled(true);
        mHud.startCameraCapture();
    }

    static public void StopCamera() throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.stopCameraCapture();
        mHud.setCameraEnabled(false);
    }

    static public int GetImageFromCamera(String imageFile) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mCameraSyncObject = new Object();
        mCameraSyncResult.set(false);
        mImageFile = imageFile;
        mGetNewImage=true;
        synchronized (mCameraSyncObject)
        {
            mCameraSyncObject.wait(1000);
        }
        if(mCameraSyncResult.get())
            return 0;
        else
            return 1;
    }

    static public void ShowText(String title, String message) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.showMessage(title, message);
    }

    static public void ShowView(View v) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        Bitmap b = Bitmap.createBitmap( v.getLayoutParams().width, v.getLayoutParams().height, Bitmap.Config.ARGB_8888);
        Canvas c = new Canvas(b);
        //v.layout(v.getLeft(), v.getTop(), v.getRight(), v.getBottom());
        v.draw(c);

        mHud.showImage(b);
    }

    static public void ShowJIM(String jim) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.showJim(jim);
    }

    static public void ShowJIM(String jim, String imagesFolder, String backgroundImage, String baseJim) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        mHud.showJim(jim, imagesFolder, backgroundImage, baseJim);
    }

    static public void ShowImage(String imageFileName) throws Exception
    {
        if(mHud==null)
            throw new Exception("You must call Initialize first.");

        Bitmap b=null;
        Exception err=null;

        b= BitmapFactory.decodeStream(new FileInputStream(imageFileName));
        b = TryFixOrientation(b, imageFileName);

        if(b!=null)
            mHud.showImage(b);
    }

    static public void StartCasting(String server, int port) throws Exception
    {
        try
        {
            StopCasting();
        }
        catch (Exception e){}

        mCastLock.lock();
        try {
            InetSocketAddress destination_sin = new InetSocketAddress(InetAddress.getByName(server), port);
            Socket sock = new Socket();
            sock.connect(destination_sin);

            mCastServer = server;
            mCastPort = port;
            mCastSocket = sock;
        }
        catch (Exception e)
        {
            mCastLock.unlock();
            throw e;
        }
        mCastLock.unlock();
    }

    static public void StopCasting() throws Exception
    {
        mCastLock.lock();
        try {
            if(mCastSocket != null)
            {
                mCastSocket.close();
                mCastSocket=null;
            }
        }
        catch (Exception e)
        {
            mCastLock.unlock();
            throw e;
        }
        mCastLock.unlock();
    }

    static void CastImage(byte [] image)
    {
        //Need to do this if we use presentation mode and need to discard images
        /*if(!mCastLock.tryLock())
            return;//Still sending previous image, ignore this image*/

        mCastLock.lock();
        try {
            if(mCastSocket != null)
            {
                int len = image.length;
                String lenS = "0000000000"+String.valueOf(len);
                lenS = lenS.substring(lenS.length()-10);

                OutputStream os = mCastSocket.getOutputStream();

                //Send image size
                byte[] sizeBuff;
                sizeBuff = lenS.getBytes("UTF-8");
                os.write(sizeBuff);

                //Send data
                os.write(image);
            }
        }
        catch (Exception e)
        {
            try {
                mCastSocket.close();
            }
            catch (Exception e2){}
            try {
                StartCasting(mCastServer, mCastPort);
            }
            catch (Exception e2){}
        }
        mCastLock.unlock();
    }

    static private Bitmap TryFixOrientation(Bitmap bmp, String filePath)
    {
        try
        {
            ExifInterface exif = new ExifInterface(filePath);
            int orientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_UNDEFINED);

            android.graphics.Matrix matrix = new android.graphics.Matrix();
            switch (orientation) {
                case ExifInterface.ORIENTATION_ROTATE_90:
                    matrix.postRotate(90);
                    break;
                case ExifInterface.ORIENTATION_ROTATE_180:
                    matrix.postRotate(180);
                    break;
                case ExifInterface.ORIENTATION_ROTATE_270:
                    matrix.postRotate(270);
                    break;
                default:
                    return bmp;
            }

            Bitmap mBitmap = Bitmap.createBitmap(bmp, 0, 0, bmp.getWidth(), bmp.getHeight(), matrix, true);
            if(mBitmap!=null)
                return mBitmap;
            else
                return bmp;
        }
        catch (Exception e)
        {
            return bmp;
        }
        catch (OutOfMemoryError e)
        {
            return bmp;
        }
    }
}
